! nc_error.f90 - object-oriented netCDF error interface (error)
! vi: set sw=4:
! Copyright (C) TOYODA Eizi, 2000.  All rights reserved.
!
! 2000-01-11 Lcpi	

module netcdf_error
    use netcdf_v3

    character(len=*), private, parameter:: RCSID = "$Id:$"

    integer, parameter:: NC_ERROR_STRLEN = 80
    integer, parameter:: NC_ERROR_NAMELEN = 32

    type NC_ERROR
    	integer::			status
	character(len=NC_ERROR_NAMELEN)	where
	character(len=NC_ERROR_STRLEN)	cause
    end type

    type(NC_ERROR), parameter:: NC_ERROR_INIT = &
	& NC_ERROR(0, ' ', ' ')

    type(NC_ERROR), save:: nc_error_saved = NC_ERROR_INIT

    interface operator(.error.)
	module procedure NetcdfErrorHappened
    end interface

    interface operator(.eq.)
	module procedure NetcdfEquivalentError
    end interface

    interface assignment(=)
	module procedure NetcdfSetErrorInt
    end interface

    interface Message
    	module procedure NetcdfMessageError
    end interface

    interface Assert
    	module procedure NetcdfAssertError
    end interface

    interface NetcdfSaveError
	module procedure NetcdfSaveErrorInt, NetcdfSaveErrorText
    end interface

contains

    ! bZ[W󎚂ɓK`̕ɕϊ
    function NetcdfMessageError(error) result(return)
    	character(len=NC_ERROR_STRLEN)::	return
    	type(NC_ERROR), intent(in)::		error
    continue
	return = ''
	if (len_trim(error%where) > 0) then
	    return = '(' // trim(error%where) // ')'
	endif
	if (len_trim(error%cause) > 0) then
	    return = trim(return) // ' ' // trim(error%cause) // ':'
	endif
	return = trim(return) // ' ' // nf_strerror(error%status)
    end function

    logical function NetcdfErrorHappened(error)
    	type(NC_ERROR), intent(in)::	error
    continue
	NetcdfErrorHappened = (error%status /= NF_NOERR)
    end function

    logical function NetcdfEquivalentError(error, status)
    	type(NC_ERROR), intent(in)::	error
    	integer, intent(in)::			status
    continue
	NetcdfEquivalentError = (error%status == status)
    end function

    subroutine NetcdfSetErrorInt(error, status)
    	type(NC_ERROR), intent(out)::	error
    	integer, intent(in)::			status
    continue
    	error%status = status
	error%where = ''
	error%cause = '(unknown cause)'
    end subroutine

    subroutine NetcdfAssertError(error)
    	type(NC_ERROR), intent(in)::	error
    continue
	if (.not..error. error) return
	print *, 'netCDF Error', trim(Message(error))
	stop 'stop'
    end subroutine

    subroutine NetcdfSaveErrorText(status, where, cause)
	integer, intent(in)::			status
	character(len=*), intent(in)::		where
	character(len=*)::			cause
    continue
	if (status == NF_NOERR) return
	nc_error_saved = NC_ERROR(status, where, cause)
    end subroutine

    subroutine NetcdfSaveErrorInt(status, where, cause)
	integer, intent(in)::			status
	character(len=*), intent(in)::		where
	integer, intent(in)::			cause
	character(len=NC_ERROR_STRLEN)::	causestring
    continue
	if (status == NF_NOERR) return
	write(causestring, *) 'id:', cause
	nc_error_saved = NC_ERROR(status, where, causestring)
    end subroutine

end module

